<?php
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html

require_once('libext/parse_ini_file2.php');

/********************************************************************************
    Entry point for yurt computation
    
    @license    GPL
    @author     <a href='http://www.tig12.net'>Thierry Graff</a>
    @history    2007.05.26, 01h30 : Creation
********************************************************************************/

class Yurt{
    
    /** The characteristics of this yurt, retrieved from config file **/
    private $config = '';
    
    /** Data computed by yurt parts **/
    protected $results = '';
    
    //***************************************************
    /**
        Builds a Yurt object from config file.
        @param $config string - Can be either a name of a file containing the directives
            or a string containing the directives
    **/
    public function __construct($what){
        try{
            if(is_file($what)){
                $config = parse_ini_file2($what);
            }
            else{
                $config = parse_ini_string2($what);
            }
            $config['general']['ROOT_DIR'] = dirname(__FILE__);
            $this->config = $config;
            $this->checkConfig();
        }
        catch(Exception $e){
            echo '<br/>Erreur : ' .    $e->getMessage() . "\n";
            exit;
        }
    }// end __construct
    
    
    //***************************************************
    /** Used by YurtParts to store a result **/
    public function setResults($key1, $key2, $value){
        $this->results[$key1][$key2] = $value;
    }
    
    
    //***************************************************
    /** Return a result computed by a part **/
    public function getResults($key1, $key2){
        return $this->results[$key1][$key2];
    }
    
    
    //***************************************************
    /**
        Returns the characteristics of this yurt.
        Returns the complete array or part of it, depending on the values of $key1 and $key2
    **/
    public function getConfig($key1='', $key2=''){
        if($key1 == '') return $this->config;
        if($key2 == '') return $this->config[$key1];
        return $this->config[$key1][$key2];
    }
    
    
    //***************************************************
    /**
        Generates a document containing the plans of the yurt.
        Unique entry point of yurt API
        @param $outputDir The directory where the plans are saved
        @return true in case of normal execution
    **/
    public function generateDocument($outputDir){
        global $HEADER, $FOOTER;
        try{
            //
            //
            $body = ''; // body of the document
            $toc = ''; // table of contents
            //
            //
            // Prepare output dir
            if(!is_dir($outputDir)){
                if(!@mkdir($outputDir))
                    throw new Exception("Impossible de cr&eacute;er le r&eacute;pertoire $outputDir");
            }
            $config = $this->config;
            //echo "<pre>"; print_r($config); echo "</pre>";
            // compute alpha or pt
            if(isset($config['general']['pt'])){
                $config['general']['alpha'] = atan($config['general']['pt'] / 100);
                $config['general']['alpha_deg'] = $config['general']['alpha'] * 180 / M_PI;
            }
            else if(isset($config['general']['alpha_deg'])){
                $config['general']['alpha'] = $config['general']['alpha_deg'] * M_PI / 180;
                $config['general']['pt'] = 100 * atan($config['general']['alpha']);
            }
            $this->config = $config;
            
            //
            //        General
            //
            if($config['document']['general']){
                $toc .= '<a href="#general">G&eacute;n&eacute;ralit&eacute;s</a>&nbsp;';
                $body .= "<h2><a name='general'>G&eacute;n&eacute;ralit&eacute;s</a></h2>";
                require_once('yurtparts/General.php');
                $part = new General($this);
                if(!copy(dirname(__FILE__) . "/images/yourte_schema.jpg", "$outputDir/yourte_schema.jpg"))
                    throw new Exception("Impossible d'&eacute;crire dans le r&eacute;pertoire $outputDir");
                $body .= "<table style='page-break-after:always;'><tr>\n";
                $body .= "<td valign='top'>\n";
                $body .= "<div class='result'>" . $part->toHtml() . "</div>";
                $body .= "</td>\n";
                $body .= "<td valign='top'>\n";
                $body .= "<img src='yourte_schema.jpg' alt='Yourte : schema general'/>";
                $body .= "</td>\n";
                $body .= "</tr></table>\n";
            }
            
            //
            //        Plancher
            //
            if($config['document']['floor']){
                $toc .= '<a href="#floor">Plancher</a>&nbsp;';
                $body .= "<h2><a name='floor'>Plancher</a></h2>";
                require_once('yurtparts/Floor.php');
                $part = new Floor($this);
                $body .= "<table style='page-break-after:always;'><tr>\n";
                $body .= "<td valign='top'>\n";
                $img = $part->toHtml('image');
                imagepng($img, "$outputDir/floor.png");
                $body .= "<img src='floor.png' alt='Plan d\'un element de plancher'/>";
                $body .= "</td>\n";
                $body .= "<td valign='top'>\n";
                $body .= "<div class='result' style='margin-left:10px;'>" . $part->toHtml('table') . "</div>";
                $body .= "</td>\n";
                $body .= "</tr></table>\n";
            }
            
            //
            //        Perches
            //
            if($config['document']['poles']){
                $toc .= '<a href="#poles">Perches</a>&nbsp;';
                $body .= "<h2><a name='poles'>Perches</a></h2>";
                require_once('yurtparts/Poles.php');
                $poles = new Poles($this); // called $poles, not $part, because needed by LinenRoof
                if(!@copy("../src/images/schema_perches.jpg", "$outputDir/schema_perches.jpg"))
                    throw new Exception("Impossible d'&eacute;crire dans le r&eacute;pertoire $outputDir");
                $body .= "<table style='page-break-after:always;'><tr>\n";
                $body .= "<td valign='top'>\n";
                $body .= "<img src='schema_perches.jpg' alt='Yourte : schema perches'/>";
                $body .= "</td>\n";
                $body .= "</tr><tr>\n";
                $body .= "<td valign='top'>\n";
                $body .= "<div class='result'>" . $poles->toHtml() . "</div>";
                $body .= "</td>\n";
                $body .= "</tr></table>\n";
            }
            
            //
            //        Couronne
            //
            if($config['document']['crown']){
                $toc .= '<a href="#crown">Couronne</a>&nbsp;';
                $body .= "<h2><a name='crown'>Couronne</a></h2>";
                require_once('yurtparts/Crown.php');
                $part = new Crown($this);
                // main
                if(!@copy("../src/images/couronne_details.jpg", "$outputDir/couronne_details.jpg"))
                    throw new Exception("Impossible d'&eacute;crire 'couronne_details.jpg' dans le r&eacute;pertoire <b>$outputDir</b>");
                // plank
                $img = $part->toHtml('plank');
                imagepng($img, "$outputDir/crown_plank.png");
                
                $body .= "<table><tr>\n";
                $body .= "<td>\n";
                $body .= "<div class='result'>" . $part->toHtml('main') . "</div>";
                $body .= "</td>\n";
                $body .= "<td style='vertical-align:bottom;'>\n";
                $body .= "<img src='couronne_details.jpg' alt='Yourte : detail d'un trou dans la couronne'/>";
                $body .= "</td>\n";
                $body .= "</tr></table>\n";
                
                $body .= '<h2>Couronne : mesures pour d&eacute;couper une planche</h2>';
                $body .= "<img src='crown_plank.png' alt='Mesures pour la decoupe d\'une planche de la couronne'/>";
                $body .= "<div style='page-break-after:always;'></div>";
            }
            
            //
            //        Treillis
            //
            if($config['document']['walls']){
                $toc .= '<a href="#walls">Treillis</a>&nbsp;';
                $body .= "<h2><a name='walls'>Treillis</a></h2>";
                require_once('yurtparts/Walls.php');
                $part = new Walls($this);
//                $body .= "<div class='result'>" . $part->toHtml() . "</div>";
                if(!@copy("../src/images/img_yourte_treillis3.jpg", "$outputDir/img_yourte_treillis3.jpg"))
                    throw new Exception("Impossible d'&eacute;crire 'img_yourte_treillis3.jpg' dans le r&eacute;pertoire <b>$outputDir</b>");
                if(!@copy("../src/images/img_yourte_treillis4.jpg", "$outputDir/img_yourte_treillis4.jpg"))
                    throw new Exception("Impossible d'&eacute;crire 'img_yourte_treillis4.jpg' dans le r&eacute;pertoire <b>$outputDir</b>");
                if(!@copy("../src/images/img_yourte_treillis5.jpg", "$outputDir/img_yourte_treillis5.jpg"))
                    throw new Exception("Impossible d'&eacute;crire 'img_yourte_treillis5.jpg' dans le r&eacute;pertoire <b>$outputDir</b>");
                
                $body .= "<table><tr>\n";
                $body .= "  <td valign='top'>\n";
                $body .= "      <div class='result'>" . $part->toHtml('main') . "</div>";
                $body .= "  </td>\n";
                $body .= "  <td valign='top'>\n";
                $body .= "      <table>\n";
                $body .= "      <tr>\n";
                $body .= "          <td valign='top'>\n";
                $body .= "              <img src='img_yourte_treillis3.jpg' alt='Yourte : détail treillis croisillon'/>";
                $body .= "          </td>\n";
                $body .= "          <td valign='top' rowspan='2'>\n";
                $body .= "              <img src='img_yourte_treillis5.jpg' alt='Yourte : détail perche sur treillis'/>";
                $body .= "          </td>\n";
                $body .= "      </tr>\n";
                $body .= "      <tr>\n";
                $body .= "          <td valign='top'>\n";
                $body .= "              <img src='img_yourte_treillis4.jpg' alt='Yourte : détail hauteurs treillis'/>";
                $body .= "          </td>\n";
                $body .= "      </tr>\n";
                $body .= "      </table>\n";
                $body .= "  </td>\n";
                $body .= "</tr></table>\n";
                
                

            }
            
            //
            //        Toile du toit
            //
            if($config['document']['linen_roof']){
                $toc .= '<a href="#linen_roof">Toile du toit</a>&nbsp;';
                $body .= "<h2><a name='linen_roof'>Toile du toit</a></h2>";
                require_once('yurtparts/Roof.php');
                // to create a Roof object, we must have a Poles object, for $lp.
                if(!isset($poles)){
                    throw new Exception("Besoin des perches pour calculer la toile du toit");
                }
                $part = new Roof($this);
                // main
                if(!@copy("../src/images/forme_toit_yourte.jpg", "$outputDir/forme_toit_yourte.jpg"))
                    throw new Exception("Impossible d'&eacute;crire 'forme_toit_yourte.jpg' dans le r&eacute;pertoire $outputDir");
                if(!@copy("../src/images/toile_toit3.jpg", "$outputDir/toile_toit3.jpg"))
                    throw new Exception("Impossible d'&eacute;crire 'toile_toit3.jpg' dans le r&eacute;pertoire $outputDir");
                if(!@copy("../src/images/toile_toit6.jpg", "$outputDir/toile_toit6.jpg"))
                    throw new Exception("Impossible d'&eacute;crire 'toile_toit6.jpg' dans le r&eacute;pertoire $outputDir");
                $body .= "<table style='page-break-after:always;'><tr>\n";
                $body .= "<td valign='top'>\n";
                $body .= "<div class='result'>" . $part->toHtml() . "</div>";
                $body .= "</td>\n";
                $body .= "<td valign='top'>\n";
                $body .= "<img src='forme_toit_yourte.jpg'
                                alt='Forme generale du toit'
                                style='float:left; position:relative; top:161px; padding-right:1em;'/>";
                $body .= "<img src='toile_toit3.jpg' alt='Toile du toit : schema des longueurs'/>";
                $body .= "<br/><br/>";
                $body .= "<img src='toile_toit6.jpg' alt='Toile du toit : schema de l'assemblage des bandes'/>";
                $body .= "</td>\n";
                $body .= "</tr></table>\n";
            }
            
            //
            //        Page layout
            //
                $body = 
                                $HEADER
                                . (isset($config['document']['title']) ? "<h1>{$config['document']['title']}</h1>" : '')    
                            . "<div id='toc'>$toc</div>"
                            . $body
                            . $FOOTER;
            //
            //
            if(!@copy("../src/css/style.css", "$outputDir/style.css"))
                throw new Exception("Impossible d'&eacute;crire 'style.css' dans le r&eacute;pertoire $outputDir");
            file_put_contents("$outputDir/index.html", $body);
            header("$outputDir/index.html");
        }
        catch(Exception $e){
            echo '<br/>Erreur Yurt.php line ' . $e->getLine() . ' - ' . $e->getMessage() . "\n";
            exit;
        }
    }// end generateDocument
    
    //***************************************************
    /**
        Checks the validity of $this->config
        @throws Exception if errors found in config
    **/
    private function checkConfig(){
        $msg = '';
        // check document section
        // incoherent value => set to false ; no error generated
        $vars = array('general', 'floor', 'walls', 'crown', 'poles', 'linen_roof');
        foreach($vars as $var){
            if(!isset($this->config['document'][$var])){
                $this->config['document'][$var] = false;
            }
        }
        // check general section - required for all parts
        // NOTE: not all values are necessary for all parts ; check could be more permissive
// @todo fix pt / alpha_deg
        $section = 'general';
        $vars = array('D', 'dc', 'h', 'pt', 'np', 'eiv');
        $msg .= $this->checkSection($section, $vars);
        // check floor section
        $section = 'floor';
        if($this->config['document'][$section]){
            $vars = array('R1', 'n', 'pm', 'scale', 'th', 'hgap1', 'hgap2', 'vgap1', 'vgap2', 'fs', 'fs2', 'fs3', 'pointSize');
            $msg .= $this->checkSection($section, $vars);
        }
        // check walls section
        $section = 'walls';
        if($this->config['document'][$section]){
            $vars = array('h1', 'h2', 'r', 'lt', 'nt', 'pst', 'etas', 'cam');
            $msg .= $this->checkSection($section, $vars);
        }
        // check crown section
        $section = 'crown';
        if($this->config['document'][$section]){
            $vars = array('hc', 'vc', 'wpc', 'epc', 'lpc', 'dpc', 'AM', 'cam', 'step', 'lbpc', 'hgap', 'vgap', 'fs', 'scaleV', 'scaleH', 'pointSize');
            $msg .= $this->checkSection($section, $vars);
        }
        // check poles section
        $section = 'poles';
        if($this->config['document'][$section]){
            $vars = array('lpd');
            $msg .= $this->checkSection($section, $vars);
        }
        // check linen_roof section
        $section = 'linen_roof';
        if($this->config['document'][$section]){
            $vars = array('wtt', 'wttu', 'gtt', 'cam', 'ltth', 'lttb');
            $msg .= $this->checkSection($section, $vars);
        }
        //
        if($msg){
            throw new Exception($msg);
        }
    }// end checkConfig
    
    
    // ******************************************************
    /**
        Checks the coherence of the variables of one section
        Auxiliary method of checkConfig
        @param $section ex 'floor' : the section to check
        @param $vars ex array('h1', 'h2', 'r', 'lt', 'nt', 'pst', 'etas', 'cam')
        @return An error message or empty string if section OK
    **/
    private function checkSection($section, $vars){
        $msg = '';
        foreach($vars as $var){
            if(!isset($this->config[$section][$var])){
                $msg .= "<div>- section <code>$section</code> : <b>$var</b> est obligatoire</div>";
            }
            else if(!is_numeric($this->config[$section][$var])){
                $msg .= "<div>- section <code>$section</code> : <b>$var</b> doit être numérique (le programme lit la valeur : <b>{$this->config[$section][$var]}</b>).</div>";
            }
        }
        return $msg;
    }// end checkSection
    
    
}// end class Yurt


$HEADER = <<<HEADER
        <!DOCTYPE html>
        <html lang="en">
        <head>
            <meta charset="utf-8" />
            <title>Plans de la yourte</title>
            <link href="style.css" rel="stylesheet" type="text/css"/>
        </head>
        <body>
HEADER;

$FOOTER = <<<FOOTER
        </body>
        </html>
FOOTER;

