<?php
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html
//********************************************************************************
/**
Calculs lies a la couronnne

@version  1.0
@license  GPL
@author   <a href='http://www.tig12.net'>Thierry Graff</a>
@php      PHP5
@history 2007.05.24, 04h03 : Creation
@history 2007.06.12 : transformation to a class
****************************************************************************************/
require_once(dirname(dirname(__FILE__)) . '/YurtPart.php');

class Crown extends YurtPart{
  
  /** Variable used to exchange data between main() and plank() **/
  private $gamma_deg;
  
  /**
    @param $params Can be 'main' or 'plank'
           'main' returns the html code of a table with crown's main data.
           'plank' returns an image with the measures of the plank.
  **/
  public function toHtml($params=''){
    switch($params){
      case 'main' : return $this->main();
      case 'plank' : return $this->plank();
      default : throw new Exception('Crown.toHtml() called with wrong parameter $params');
    }
  }// end toHtml
  
  
  /** Generates a table withe crown's main data **/
  private function main(){
    // Variables importees
    $D = $this->yurt->getConfig('general', 'D'); // Diametre de la yourte, en m
    $alpha = $this->yurt->getConfig('general', 'alpha'); // radians
    $np = $this->yurt->getConfig('general', 'np'); // nombre de perches fichees dans la couronne
    //
    // Valeurs a fixer a cette etape
    $lpc = $this->yurt->getConfig('crown', 'lpc'); // longueur de perche dans couronne, en m
    $dpc = $this->yurt->getConfig('crown', 'dpc'); // diametre perche dans couronne, en m
    //
    $dc = $this->yurt->getConfig('general', 'dc'); // diametre exterieur de la couronne, en m
    $hc = $this->yurt->getConfig('crown', 'hc'); // epaisseur horizontale de la couronne, en m
    $vc = $this->yurt->getConfig('crown', 'vc'); // epaisseur verticale de la couronne, en m
    // planches
    $wpc = $this->yurt->getConfig('crown', 'wpc'); // largeur des planches, en m
    $epc = $this->yurt->getConfig('crown', 'epc'); // epaisseur des planches, en m
    // detail
    $AM = $this->yurt->getConfig('crown', 'AM'); // decalage vertical perche, en m
    // cout
    $cam = $this->yurt->getConfig('crown', 'cam'); // cout au metre lineaire de planches, en euros
    
    // ************* calculs
    // generalites
    $R = $D / 2;
    $rc = $dc / 2;
    $pc = 2 * M_PI * $rc;
    // rayon interne
    $dci = $dc - 2*$hc;
    $rci = $dci / 2;
    // repartition des trous
    $delta = (2 * M_PI * $rc - $np * $dpc) / $np;
    // planches
    $gamma1 = acos(($rc - $wpc) / ($rc - $hc)); // radians
    $gamma1_deg = 180 * $gamma1 / M_PI;
    $npc1 = ceil(360/(2*$gamma1_deg));
    $gamma_deg = 180 / $npc1;
    $gamma = M_PI * $gamma_deg / 180;
    $nepc = $vc / $epc;
    $npc = $npc1 * $nepc;
    $lbpc = 2 * $rc * sin($gamma);
    $ltpc = $lbpc * $npc;
    // detail
    $PM = $dpc / cos($alpha);
    $H1N = $lpc + ($AM + $dpc) * sin($alpha);
    $AI = $AM + $dpc / (2 * cos($alpha));
    $IJ = $lpc + $dpc / 2 *tan($alpha);
    $PD = $vc - $PM - $AM;
    $OH3 = $PD - $lpc * sin($alpha);
    $AH7 = $dpc * sin($alpha) + $lpc * cos($alpha);
    // cout
    $coutPlanche = $cam * $lbpc;
    $coutTotal = $cam * $ltpc;

    // Affichage des resultats
    $res = '';
    $res .= "<table>\n";
    $res .= "<tr><th colspan='2'><b>Variables import&eacute;es</b></th></tr>\n";
    $res .= "<tr><td>Diam&egrave;tre perches dans couronne</td><td><b>dpc = " . round(self::m2cm($dpc), 2) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Longueur de perche dans couronne</td><td><b>lpc = " . round(self::m2cm($lpc), 2) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Angle du toit</td><td><b>&alpha; = " . round($alpha*180/M_PI, 2) . " &deg;</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>Couronne</th></tr>\n";
    $res .= "<tr><td>Diam&egrave;tre ext&eacute;rieur</td><td><b>dc = dce = " . round($dc, 2) . " m</b></td></tr>\n";
    $res .= "<tr><td>Rayon ext&eacute;rieur</td><td><b>rc = rce = " . round($rc, 2) . " m</b></td></tr>\n";
    $res .= "<tr><td>P&eacute;rim&egrave;tre ext&eacute;rieur</td>
                 <td><b>pce = " . round($pc, 2) . " m</b>
             </tr>\n";
    $res .= "<tr><td>Diam&egrave;tre int&eacute;rieur</td><td><b>dci = " . round($dci, 2) . " m</b></td></tr>\n";
    $res .= "<tr><td>Rayon int&eacute;rieur</td><td><b>rci = " . round($rci, 2) . " m</b></td></tr>\n";
    $res .= "<tr><td>Epaisseur horizontale</td><td><b>hc = " . round(self::m2cm($hc), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Epaisseur verticale</td><td><b>vc = " . round(self::m2cm($vc), 1) . " cm</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>Planches</th></tr>\n";
    $res .= "<tr><td>Largeur des planches</td><td><b>wpc = " . round(self::m2cm($wpc), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Epaissseur des planches</td><td><b>epc = " . round(self::m2cm($epc), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Nombre d'&eacute;tages de planches</td><td><b>nepc = " . $nepc . "</b></td></tr>\n";
    $res .= "<tr><td>Angle interm&eacute;diaire</td><td><b>&gamma;<sub>1</sub> = " . round($gamma1_deg, 1) . " &deg;</b></td></tr>\n";
    $res .= "<tr><td>Angle retenu</td><td><b>&gamma; = " . round($gamma_deg, 1) . " &deg;</b></td></tr>\n";
    $res .= "<tr><td>Nombre de bouts de planche par &eacute;tage</td><td><b>npc1 = " . $npc1 . "</b></td></tr>\n";
    $res .= "<tr><td>Nombre total de bouts de planche</td><td><b>npc = " . $npc . "</b></td></tr>\n";
    $res .= "<tr><td>Longueur d'un bout de planche</td><td><b>lbpc = " . round(self::m2cm($lbpc)) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Longueur totale de planche n&eacute;cessaire</td><td><b>ltpc = " . round($ltpc, 2) . " m</b></td></tr>\n";
    $res .= "<tr><td>Co&ucirc;t au m&egrave;tre de planche</td><td><b>cam = " . $cam . " euros</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>Perches dans la couronne</th></tr>\n";
    $res .= "<tr><td>Nombre de perches</td><td><b>np = $np</b></td></tr>\n";
    $res .= "<tr><td>Diam&egrave;tre ext&eacute;rieur couronne</td><td><b>dc = dce = " . round($dc, 2) . " m</b></td></tr>\n";
    $res .= "<tr><td>Diam&egrave;tre int&eacute;rieur couronne</td><td><b>dci = " . round($dci, 2) . " m</b></td></tr>\n";
    $res .= "<tr><td>P&eacute;rim&egrave;tre ext&eacute;rieur couronne</td>
                 <td><b>pce = " . round($pc, 2) . " m</b>
                 <br/><b>pc / 360 = " . round(self::m2cm($pc/360), 2) . " cm</b>
                 <br/><b>pc / np = " . round(self::m2cm($pc/$np), 2) . " cm</b></td>
             </tr>\n";
    $res .= "<tr><td>Diam&egrave;tre perche dans couronne</td><td><b>dpc = " . round(self::m2cm($dpc), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Espace entre deux trous</td><td><b>dit = " . round(self::m2cm($delta), 1) . " cm</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>D&eacute;tails d'un trou de perche</th></tr>\n";
    $res .= "<tr><td>D&eacute;calage - choix effectu&eacute;</td><td><b>AM = " . round(self::m2cm($AM), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Longueur perche &agrave; tailler au bon diam&egrave;tre</td><td><b>H<sub>1</sub>N = lpc2 = " . round(self::m2cm($H1N), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Position du trou</td><td><b>AI = " . round(self::m2cm($AI), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Profondeur du trou</td><td><b>IJ = " . round(self::m2cm($IJ), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Pour l'isolant</td><td><b>PD = " . round(self::m2cm($PD), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Pour voir solidit&eacute;</td><td><b>OH<sub>3</sub> = " . round(self::m2cm($OH3), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Profondeur du trou</td><td><b>AH<sub>7</sub> = " . round(self::m2cm($AH7), 1) . " cm</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>Prix</th></tr>\n";
    $res .= "<tr><td>Co&ucirc;t planches au m&egrave;tre</td><td><b>cam = " . round($cam, 2) . " euros</b></td></tr>\n";
    $res .= "<tr><td>Co&ucirc;t d'un &eacute;l&eacute;ment</td><td><b>coutPlanche = " . round($coutPlanche, 2) . " euros</b></td></tr>\n";
    $res .= "<tr><td>Co&ucirc;t total</td><td><b>coutTotal = " . round($coutTotal, 2) . " euros</b></td></tr>\n";
    $res.= "</table>\n";
    
    return $res;
  }// end main
  
  /** Generates an image with the measures to cut a round piece of wood **/
  private function plank(){
//echo "<pre>"; print_r($this->data); echo "</pre>";
    // Valeurs importees, a  fixer par l'utilisateur
    $dc = $this->yurt->getConfig('general', 'dc'); // diametre exterieur de la couronne, en m
    $hc = $this->yurt->getConfig('crown', 'hc'); // epaisseur horizontale de la couronne, en m
    $gamma_deg = $this->gamma_deg;
    $step = $this->yurt->getConfig('crown', 'step'); // pas en degres
    $lbpc = $this->yurt->getConfig('crown', 'lbpc'); // longueur d'un bout de planche, en cm
    $wpc = $this->yurt->getConfig('crown', 'wpc'); // largeur d'un bout de planche, en cm
    $font = $this->yurt->getConfig('general', 'ROOT_DIR') . '/fonts/ARIAL.TTF';
    $hgap = $this->yurt->getConfig('crown', 'hgap'); // espace inutile laisse au bord gauche de l'image
    $vgap = $this->yurt->getConfig('crown', 'vgap'); // espace inutile laisse au bord haut de l'image
    $fs = $this->yurt->getConfig('crown', 'fs'); // font size
    $scaleV = $this->yurt->getConfig('crown', 'scaleV'); // facteur d'echelle vertical; 1 cm <--> $scaleV px
    $scaleH = $this->yurt->getConfig('crown', 'scaleH'); // facteur d'echelle horizontal ; 1 cm <--> $scaleH px
    $pointSize = $this->yurt->getConfig('crown', 'pointSize');
    
    // calculs
    $wpc_m = $wpc; // wpc in m
    $wpc *= 100; // convert to cm
    $hgap += 2*$fs;
    $vgap += 2*$fs;
    $rc = $dc / 2;
    $dci = $dc - 2*$hc;
    $rci = $dci / 2;
    // On recalcule gamma (deja fait dans main())
    $gamma1 = acos(($rc - $wpc_m) / ($rc - $hc)); // radians
    $gamma1_deg = 180 * $gamma1 / M_PI;
    $npc1 = ceil(360/(2*$gamma1_deg));
    $gamma_deg = 180 / $npc1;
    $gamma = M_PI * $gamma_deg / 180;
    // Calcul des coordonnees
    $n = floor($gamma_deg / $step);
    for($i=0; $i <= $n; $i++){
      $gammai_deg = $i * $step;
      $gammai = $gammai_deg * M_PI / 180;
      // Coord des points Ai et Bi
      $xA = round($rc * sin($gammai) * 100, 1);
      $yA = round(($rc * cos($gammai) - $rci * cos($gamma)) * 100, 1);
      $xB = round($rci * sin($gammai) * 100, 1);
      $yB = round($rci * (cos($gammai) - cos($gamma)) * 100, 1);
      // transformation pour mesures pratiques
      $data[$n - $i]['gammai_deg'] = $gamma_deg - $gammai_deg;
      $data[$i + $n]['gammai_deg'] = $gamma_deg + $gammai_deg;
      $data[$n - $i]['xA'] = $lbpc/2 - $xA;
      $data[$i + $n]['xA'] = $lbpc/2 + $xA;
      $data[$n - $i]['yA'] = $yA;
      $data[$i + $n]['yA'] = $yA;
      $data[$n - $i]['xB'] = $lbpc/2 - $xB;
      $data[$i + $n]['xB'] = $lbpc/2 + $xB;
      $data[$n - $i]['yB'] = $yB;
      $data[$i + $n]['yB'] = $yB;
    }
    // Dimensions de l'image
    $w = $lbpc * $scaleH + 2*$hgap;
    $h = $wpc * $scaleV + 2*$vgap;
    // Creation de l'image
    $img = imagecreate($w, $h);
    $white = imagecolorallocate($img, 255, 255, 255);
    $black = imagecolorallocate($img, 0, 0, 0);
    // Pour les lignes pointille
    $style = array($black, $black, $black, $black, $black, $white, $white, $white, $white, $white);
    imagesetstyle($img, $style);
    
    for($i=0; $i <= 2*$n; $i++){
      // dessin de Ai
      $x = $data[$i]['xA'] * $scaleH + $hgap;
      $y = ($wpc - $data[$i]['yA']) * $scaleV + $vgap;
      imagefilledellipse($img, $x, $y, $pointSize, $pointSize, $black);
      imageLine($img, $x, $y, $x, $vgap-$fs+2, IMG_COLOR_STYLED);
      imagettftext($img, $fs, 0, $x-$fs, $vgap-$fs, $black, $font, round($data[$i]['xA'], 1));
      imagettftext($img, $fs, 0, $x+$pointSize+1, $y+$fs/2, $black, $font, round($data[$i]['yA'], 1));
      // dessin de Bi
      $x = $data[$i]['xB'] * $scaleH + $hgap;
      $y = ($wpc - $data[$i]['yB']) * $scaleV + $vgap;
      imagefilledellipse($img, $x, $y, $pointSize, $pointSize, $black);
      imageLine($img, $x, $y, $x, $h-5-$fs-2, IMG_COLOR_STYLED);
      imagettftext($img, $fs, 0, $x-$fs, $h-5, $black, $font, round($data[$i]['xB'], 1));
      imagettftext($img, $fs, 0, $x+$pointSize+1, $y+$fs/2, $black, $font, round($data[$i]['yB'], 1));
    }
    // Dessin des bords
    $x1 = $data[0]['xA'] * $scaleH + $hgap;
    $y1 = ($wpc - $data[0]['yA']) * $scaleV + $vgap;
    $x2 = $data[0]['xB'] * $scaleH + $hgap;
    $y2 = ($wpc - $data[0]['yB']) * $scaleV + $vgap;
    imageLine($img, $x1, $y1, $x2, $y2, $black);
    $x1 = $data[2*$n]['xA'] * $scaleH + $hgap;
    $y1 = ($wpc - $data[2*$n]['yA']) * $scaleV + $vgap;
    $x2 = $data[2*$n]['xB'] * $scaleH + $hgap;
    $y2 = ($wpc - $data[2*$n]['yB']) * $scaleV + $vgap;
    imageLine($img, $x1, $y1, $x2, $y2, $black);
    return $img;
  }// end plank
  
  
}// end class Crown

?>