<?php
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html
//********************************************************************************
/**
Calcul de la longueur des perches

@license  GPL
@author   <a href='http://www.tig12.net'>Thierry Graff</a>
@php      PHP5
@history 2007.05.24, 0h40 : Creation
@history 2007.06.12 : transformation to a class
****************************************************************************************/
require_once(dirname(dirname(__FILE__)) . '/YurtPart.php');

class Poles extends YurtPart{
  
  public function toHtml($params=''){
    $D = $this->yurt->getConfig('general', 'D'); // rayon de la yourte, en m
    $dc = $this->yurt->getConfig('general', 'dc'); // rayon de la couronne, en m
    $etas = $this->yurt->getConfig('walls', 'etas'); // epaisseur des tasseaux du treillis, en m
    
    $alpha = $this->yurt->getConfig('general', 'alpha'); // angle du toit, en rad
    $eiv = $this->yurt->getConfig('general', 'eiv'); // epaisseur de l'isolation verticale, en m
    $lpc = $this->yurt->getConfig('crown', 'lpc'); // longueur de perche dans la couronne, en m
    $dpc = $this->yurt->getConfig('crown', 'dpc'); // diametre de perche dans la couronne, en m --- inutile pour le calcul, juste pour affichage
    $lpd = $this->yurt->getConfig('poles', 'lpd'); // longueur de perche qui depasse, en m
  
    // calculs, en m
    //$alpha = $alpha * M_PI / 180; // radians
    $R = $D / 2;
    $rc = $dc / 2;
    $lp1 = ($R - $rc) / cos($alpha);
    $lpi = $eiv / cos($alpha);
    $lpt = $etas / cos($alpha);
    $lp = $lpc + $lp1 + $lpi + $lpd;
    // Store $lp in $results because $lp can be used by Roof 
    $this->yurt->setResults('poles', 'lp', $lp);
    
    // Affichage des resultats
    $res = '';
    $res .= "<table>\n";
    //
    $res .= "<tr><th colspan='2'>Longueurs</th></tr>\n";
    $res .= "<tr><td><b>Longueur totale d'une perche</b></td><td><b>lp = "
         . round(self::m2cm($lp), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Longueur de perche dans la couronne</td><td><b>lpc = "
         . round(self::m2cm($lpc), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Diam&egrave;tre d'une perche dans la couronne</td><td><b>dpc = "
         . round(self::m2cm($dpc), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Longueur entre couronne et treillis</td><td><b>lp1 = "
         . round(self::m2cm($lp1), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Longueur sur l'&eacute;paisseur du treillis</td><td><b>lpt = "
         . round(self::m2cm($lpt), 1) . " cm</b>"
         . "<br/>2 * lpt = " . round(self::m2cm(2*$lpt), 1) . " cm</td></tr>\n";
    $res .= "<tr><td>Longueur sur isolant vertical</td><td><b>lpi = "
         . round(self::m2cm($lpi), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Longueur de d&eacute;passement</td><td><b>lpd = "
         . round(self::m2cm($lpd), 1) . " cm</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>Trous</th></tr>\n";
    $res .= "<tr><td>Trou en E</td><td><b>"
         . round(self::m2cm($lpd), 1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Trou en D</td><td><b>"
         . round(self::m2cm($lpd + $lpi), 1) . " cm</b></td></tr>\n";
    $res .= "</table>\n";
    return $res;
  }// end toHtml
  
}// end class Poles


?>