<?php
// Software released under the General Public License (version 2 or later), available at
// http://www.gnu.org/copyleft/gpl.html
//********************************************************************************
/**
Calcul du treillis

@version  1.0
@license  GPL
@author   <a href='http://www.tig12.net'>Thierry Graff</a>
@php      PHP5
@history 2007.05.07, 11h46 : Creation
@history 2007.06.12 : transformation to a class
****************************************************************************************/
require_once(dirname(dirname(__FILE__)) . '/YurtPart.php');

class Walls extends YurtPart{
  
  public function toHtml($params=''){
    // Valeurs importees
    $D = $this->yurt->getConfig('general', 'D'); // diametre de la yourte, en metres
    $h = $this->yurt->getConfig('general', 'h'); // hauteur de la yourte, en metres
    $np = $this->yurt->getConfig('general', 'np'); // nombre de perches
    //
    // Valeurs &agrave; fixer &agrave; cette etape
    $h1 = $this->yurt->getConfig('walls', 'h1'); // depassement en haut, en metres
    $h2 = $this->yurt->getConfig('walls','h2'); // depassement en bas, en metres
    $r = $this->yurt->getConfig('walls','r'); // Rayon du cercle fait par le treillis
    $nt = $this->yurt->getConfig('walls','nt'); // Nombre de losanges verticaux (doit &ecirc;tre un entier)
    $pst = $this->yurt->getConfig('walls','pst'); // longueur sans treillis, en metres
    $cam = $this->yurt->getConfig('walls','cam'); // cout des tasseaux, au metre.
    
    // calculs
    $pdt = 2 * M_PI * $r;
    $htl = $h - $h1 - $h2;
    $ht = $pdt / $np;
    $vt = $htl / $nt;
    $lt = sqrt(pow($vt, 2) + pow($ht, 2))/2;
    $alphat = 2 * atan($ht / $vt); // radians
    $pat = $pdt - $pst;
    $ltas = 2 * $lt * $nt + ($h1 + $h2)/cos($alphat / 2);
    $ntas = 2 * $pat / $ht;
    $ni = floor($ntas * (2*$nt + 1));
    $lttas = $ntas * $ltas; // longueur totale des tasseaux
    $coutTasseau = $cam * $ltas;
    $coutTotal = $cam * $lttas;
    
    // Affichage des resultats
    $res = '';
    $res .= "<table>\n";
    $res .= "<tr><th colspan='2'>Donn&eacute;es de d&eacute;part</th></tr>\n";
    $res .= "<tr><td>Diam&egrave;tre de la yourte</td><td><b>D = " . self::formatMeters($D, 2) . "</b></td></tr>\n";
    $res .= "<tr><td>Hauteur de la yourte</td><td><b>h = " . self::formatMeters($h, 2) . "</b></td></tr>\n";
    $res .= "<tr><td>Nombre de perches</td><td><b>np = " . $np . "</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>P&eacute;rim&egrave;tres</th></tr>\n";
    $res .= "<tr><td>P&eacute;rim&egrave;tre avec treillis</td><td><b>pat = " . self::formatMeters($pat) . "</b></td></tr>\n";
    $res .= "<tr><td>P&eacute;rim&egrave;tre du treillis</td><td><b>pdt = " . self::formatMeters($pdt) . "</b></td></tr>\n";
    $res .= "<tr><td>P&eacute;rim&egrave;tre sans treillis</td><td><b>pst = " . self::formatMeters($pst) . "</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>Losanges</th></tr>\n";
    
    $res .= "<tr><td>Nombre de losanges verticaux</td><td><b>n<sub>t</sub> = " . $nt . "</b></td></tr>\n";
    $res .= "<tr><td>D&eacute;passement haut</td><td><b>h<sub>1</sub> = " . self::m2cm($h1) . " cm</b></td></tr>\n";
    $res .= "<tr><td>D&eacute;passement bas</td><td><b>h<sub>2</sub> = " . self::m2cm($h2) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Hauteur de n<sub>t</sub> losanges</td><td><b>h<sub>tl</sub> = " . self::formatMeters($htl) . "</b></td></tr>\n";
    $res .= "<tr><td>Largeur d'un losange</td><td><b>h<sub>t</sub> = " . self::m2cm(round($ht, 3)) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Hauteur d'un losange</td><td><b>v<sub>t</sub> = " . self::m2cm(round($vt, 3)) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Longueur du c&ocirc;t&eacute; d'un losange</td><td><b>l<sub>t</sub> = " . self::m2cm(round($lt, 3)) . " cm</b></td></tr>\n";
    $res .= "<tr><td>Angle d'un losange</td><td><b>&alpha;<sub>t</sub> = " . round($alphat*180/M_PI) . "&deg;</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>Tasseaux</th></tr>\n";
    $res .= "<tr><td>Nombre de tasseaux</td><td><b>n<sub>tas</sub> = " . round($ntas, 2) . "</b></td></tr>\n";
    $res .= "<tr><td>Nombre d'intersections</td><td><b>n<sub>n</sub> = " . $ni . "</b></td></tr>\n";
    $res .= "<tr><td>Longueur d'un tasseau</td><td><b>l<sub>tas</sub> = " . self::formatMeters($ltas) . "</b></td></tr>\n";
    $res .= "<tr><td>Longueur totale de tasseaux</td><td><b>lt<sub>tas</sub> = " . self::formatMeters($lttas) . "</b></td></tr>\n";
    //
    $res .= "<tr><th colspan='2'>Co&ucirc;t</th></tr>\n";
    $res .= "<tr><td>Co&ucirc;t des tasseaux, au m&egrave;tre</td><td><b>cam = " . $cam . " euros</b></td></tr>\n";
    $res .= "<tr><td>Co&ucirc;t d'un tasseau</td><td><b>coutTasseau = " . round($coutTasseau, 2) . " euros</b></td></tr>\n";
    $res .= "<tr><td>Co&ucirc;t total</td><td><b>coutTotal = " . round($coutTotal, 2) . " euros</b></td></tr>\n";
    $res.= "</table>\n";
    return $res;
  }// end toHtml
  
}// end class Walls
?>